/* integrands used in quadrature integration with scipy's LowLevelCallables */
#include <math.h>

/* FFHS functions */
double ffhsSD(double x, void *user_data) {
	double *c = (double *)user_data;

	double omega = c[0];
	double tau = c[1];

	double u = x*x;
	double res = u*u * tau;

	res /= 81. + 9.*u - 2.*u*u + u*u*u;
	res /= u*u + omega*omega * tau*tau;
	return res;
}

/* log-gaussian functions */
double logNormalDist(double tau, double tau0, double sigma) {
    return exp(- pow((log(tau/tau0) / sigma), 2) / 2) / sqrt(2*M_PI)/sigma;
}

double logGaussianSD_high(double u, void *user_data) {
	double *c = (double *)user_data;

	double omega = c[0];
	double tau = c[1];
	double sigma = c[2];

    double uu = exp(-u);
    double dist = logNormalDist(1./uu, tau, sigma);

    return dist * omega * uu / (pow(uu, 2) + pow(omega, 2));
}

double logGaussianSD_low(double u, void *user_data) {
	double *c = (double *)user_data;

	double omega = c[0];
	double tau = c[1];
	double sigma = c[2];

    double uu = exp(u);

    double dist = logNormalDist(uu, tau, sigma);

    return dist * omega * uu / (1. + pow(omega*uu, 2));
}

double logGaussianCorrelation(double x, void *user_data) {
    double *c = (double *)user_data;

    double t = c[0];
	double tau = c[1];
	double sigma = c[2];

    double uu = exp(x);

    double dist = logNormalDist(uu, tau, sigma);

    return dist * exp(-t/uu);
}
